#!/usr/bin/python

# Copyright (C) 2009 Stefano Zacchiroli <zack@pps.jussieu.fr>
# License: GPL version 3 or (at your option) any later version

# Requirements:
# - python-matplotlib
# - python-numpy
# - python-stats

# Sample usage:
#
#   $ ./correlation_graph.py -b correlation-direct-strong -f png -f eps \
#       foo.direct-vs-sensitive
#
# which will generate graph files correlation-direct-strong.{eps,png}

import matplotlib
import matplotlib.pyplot as plt
import stats as st 
import pstat as ps 
import numpy as np
import optparse
import sys

usage = """Usage: %prog [OPTIONS] DATA

DATA should be textual databases mapping package names to pair of
integer values, one per column.
"""

supported_formats = set(['png', 'pdf', 'ps', 'eps', 'svg'])

def load_table(fname):
    """return a dict mapping keys to pairs (v1, v2) where the values
    come respectively from column 2 and 3 of fname
    """
    # def abort(witness):
    #     print "The two measure files are defined on different keys."
    #     print "E.g.: %s" % witness
    #     print "Abort."
    #     sys.exit(1)

    d = {}
    for line in open(fname):
        k, v1, v2 = line.split()[:3]
        d[k] = (int(v1), int(v2))
    return d

# dead code
def prepare_data(data):
    def normalize(vs):
        peek = float(max(vs))
        return map(lambda v: float(v)/peek, vs)

    pkgs = data.keys()
    pkgs.sort(cmp = lambda p1, p2: cmp(data[p1][0], data[p2][0])) # sort by m1

    vs = [ data[pkg] for pkg in pkgs ]
    v1s = normalize(map(lambda v: v[0], vs))
    v2s = normalize(map(lambda v: v[1], vs))
    m1 = np.array(v1s)
    m2 = np.array(v2s)
    
    return (m1, m2)

def measures(fname,data):

    dir_dep = []
    str_dep = []
    diff = []
    for (d,s) in data.values() :
        dir_dep.append(d)
        str_dep.append(s)
        diff.append(abs(d-s))

#    t = st.pearsonr(dir_dep,str_dep)
#    print t
#    tau = st.kendalltau(dir_dep,str_dep)
#    print tau

    dev_diff = st.lsamplestdev(diff)
    mean_diff = st.lmean(diff)
    mean_dir = st.lmean(dir_dep)
    mean_str = st.lmean(str_dep)
    var_diff = st.lstdev(diff)
    var_dir = st.lstdev(dir_dep)
    var_str = st.lstdev(str_dep)
#    print "release & str mean & dir mean & ( dir - str ) mean, str stdev & dir stdev & ( dir - str ) stdev\n" 
    print "%s & %f & %f & %f & %f & %f & %f\n" % (fname, mean_str, mean_dir, mean_diff, var_str, var_dir, var_diff)

#    rho = st.spearmanr(dir_dep,str_dep)
#    print "spearmanr correlation (rho,significance) = %s\n" % (var,dev,rho)

def plot(data, xlabel, ylabel):
    fig = plt.figure()

    ax = fig.add_subplot(111)
    dir_dep = []
    str_dep = []
    for (d,s) in data.values() :
        dir_dep.append(d)
        str_dep.append(s)


#    dir_dep = map(lambda v: v[0], data.values())
#    str_dep = map(lambda v: v[1], data.values())
    ax.plot(dir_dep, str_dep, 'ko', markersize=2)
    ax.plot(range(max(dir_dep)), 'k-')
    ax.set_yscale('log')
    ax.set_xscale('log')
    ax.grid(True)
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
#    st.lpaired(dir_dep,str_dep)
#    ax.plot(st.linregress(dir_dep,str_dep))
#    leg = ax.legend(('Median'),'upper center', shadow=True)

# the matplotlib.patches.Rectangle instance surrounding the legend
#    frame  = leg.get_frame()  
#    frame.set_facecolor('0.80')    # set the frame face color to light gray

# matplotlib.text.Text instances
#    for t in leg.get_texts():
#        t.set_fontsize('small')    # the legend text fontsize

# matplotlib.lines.Line2D instances
#    for l in leg.get_lines():
#        l.set_linewidth(1.5)  # the legend line width

    return fig

def main():
    global supported_formats

    cmdline = optparse.OptionParser(usage=usage)
    cmdline.add_option("-i", "--interactive", action="store_true",
                       dest="interactive", default=False,
                       help="Interactive mode to browse the correlation graph")
    cmdline.add_option("-f", "--format", dest="format", default=["png"],
                       action="append",
                       help="output format (one of: png, pdf, ps, eps, svg), "
                       "can be given multiple times, default: 'png'")
    cmdline.add_option("-b", "--basename", dest="basename", default="corrgraph",
                       help="basename for graph output, default: 'corrgraph'")
    cmdline.add_option("-m", "--measures", dest="measures", default=False,
                       help="compute variability/correlation measures'")

    (options, args) = cmdline.parse_args()
    options.format = set(options.format)
    if len(args) < 1:
        cmdline.error("Must specify a data file")
    if not options.format.issubset(supported_formats):
        cmdline.error("Unsupported format(s): %s" %
                      list(options.format - supported_formats))

    if not options.interactive:
        matplotlib.use('Agg')
    data = load_table(args[0])
    if options.measures :
        measures(args[-1],data)
    else :
        fig = plot(data,
                   xlabel="Direct sensitivity", ylabel="Strong sensitivity")
        if options.interactive:
            plt.show()
        else:
            for fmt in set(options.format):
                fname = "%s.%s" % (options.basename, fmt)
                plt.savefig(fname, format=fmt)
        #        print fname

if __name__ == '__main__':
    main()
